#pragma once

#include <GL/gl3w.h>
#include <vector>
#include <string>
#include <map>
#include <set>
#include "filesystem/path.h"

class ShaderBundle {
public:
    using ShaderHandle = GLuint;
    using ProgramHandle = GLuint;
    using ShaderType = GLenum;

    struct ShaderId {
        std::string name;
        ShaderType type;
        bool operator<(const ShaderId& rhs) const {
            return std::tie(name, type) < std::tie(rhs.name, rhs.type);
        }
        bool operator==(const ShaderId& rhs) const {
            return std::tie(name, type) == std::tie(rhs.name, rhs.type);
        }
    };

    struct Shader {
        ShaderHandle handle;

        int32_t hash;
    };

    struct Program {
        // public handle; 0 until the program has successfully linked
        ProgramHandle handle;

        ProgramHandle internal_handle;
    };

    ShaderBundle() = default;
    ~ShaderBundle();

    std::map<ShaderId, Shader> shader_pool;
    std::map<std::vector<ShaderId>, Program> programs;

    ProgramHandle* add_program(const std::vector<std::pair<std::string, ShaderType>>& shaders);

    ShaderBundle& recompile();
    ShaderBundle& link();

    static std::string preprocess(filesystem::path path, std::string shader);
};